/*
    user/group functions
    
    Copyright (c) 2003 - 2004 Night Productions, by Darkshadow.  All Rights Reserved.
    http://homepage.mac.com/darkshadow02/developer.htm
    darkshadow02@mac.com
    
    May be used freely, but keep my name/copyright in the header.
    
    There is NO warranty of any kind, express or implied; use at your own risk.
    Responsibility for damages (if any) to anyone resulting from the use of this
    code rests entirely with the user.
    
    --> allUsers(void) - returns an array of all users on the machine.
    --> allGroups(void) - returns an array of all groups on the machine.
    --> normalUsers(void) - returns an array of all users whose UID is >= 500.
    --> adminUsers(void) - returns an array of all admin users.
    --> allUsersForGroup(NSString *name) - returns an array of all users for the given group.
    --> allUsersForGID(int groupId) - returns an array of all users for the given group.
    --> infoForAllUsers(void) - returns an array of dictionaries, one for each user, containing:
	    Key:		    Value:
	    User ID		    NSNumber containing uid of user
	    User Name		    NSString containing user's name
	    User Real Name	    NSString containing user's real name
	    User Home		    NSString containing user's home directory (path)
	    User Shell		    NSString containing user's default shell
	    User Group		    NSString containing name of the user's group
	    User Group ID	    NSNumber containing gid of user's group
	    --next two may be empty arrays--
	    User Other Groups       NSArray containing NSStrings of groups user belongs to
	    User Other Group IDs    NSArray containing NSNumbers of group IDs user belongs to
	    
	    *Note: there's a one-to-one correspondence between the last two arrays.
		   i.e. the first element in each array is pointing to the same group, etc, etc.
    
    --> realNameForUser(NSString *name) - returns an NSString with the given user's real
	name.
    --> realNameForUID(int userId) - returns an NSString with the given user's real name.
    --> infoForUID(int userId) - returns an NSDictionary containing user info (see above).
    --> infoForUser(NSString *name) - returns an NSDictionary containing user info.
*/

#import <Foundation/Foundation.h>
#import <sys/types.h>
#import <pwd.h>
#import <grp.h>
#import <string.h>

#define ADMIN_GROUP 80

NSArray *allUsers(void)
{
    struct passwd *pw=NULL;
    NSMutableArray *returnArray=[[NSMutableArray alloc]initWithCapacity:1];
    
    while ((pw=getpwent()) != NULL) {
	[returnArray addObject:[NSString stringWithUTF8String:pw->pw_name]];
    }
    endpwent();
    
    return [returnArray autorelease];
}

NSArray *allGroups(void)
{
    struct group *grp=NULL;
    NSMutableArray *returnArray=[[NSMutableArray alloc]initWithCapacity:1];
    
    while ((grp=getgrent()) != NULL) {
	[returnArray addObject:[NSString stringWithUTF8String:grp->gr_name]];
    }
    endgrent();
    
    return [returnArray autorelease];
}

NSArray *normalUsers(void)
{
    struct passwd *pw=NULL;
    NSMutableArray *returnArray=[[NSMutableArray alloc]initWithCapacity:1];
    
    while ((pw=getpwent()) != NULL) {
	if ((long)pw->pw_uid >= 500) {
	    [returnArray addObject:[NSString stringWithUTF8String:pw->pw_name]];
	}
    }
    endpwent();
    
    return [returnArray autorelease];
}

NSArray *adminUsers(void)
{
    NSMutableArray *returnArray=[[NSMutableArray alloc]initWithCapacity:1];
    struct group *grp=NULL;
    char *groupMem=NULL;
    int i=0;
    
    grp=getgrgid(ADMIN_GROUP);
    
    if (grp != NULL) {
	while (((groupMem=grp->gr_mem[i]) != NULL)) {
	    [returnArray addObject:[NSString stringWithUTF8String:groupMem]];
	    i++;
	}
    }
    endgrent();
    
    return [returnArray autorelease];
}

NSArray *allUsersForGroup(NSString *name)
{
    NSMutableArray *returnArray=[[NSMutableArray alloc]initWithCapacity:1];
    struct group *grp=NULL;
    char *groupMem=NULL;
    int i=0;
    
    grp=getgrnam([name UTF8String]);
    
    if (grp != NULL) {
	while (((groupMem=grp->gr_mem[i]) != NULL)) {
	    [returnArray addObject:[NSString stringWithUTF8String:groupMem]];
	    i++;
	}
    }
    endgrent();
    
    return [returnArray autorelease];
}

NSArray *allUsersForGID(int groupId)
{
    NSMutableArray *returnArray=[[NSMutableArray alloc]initWithCapacity:1];
    struct group *grp=NULL;
    char *groupMem=NULL;
    int i=0;
    
    grp=getgrgid(groupId);
    
    if (grp != NULL) {
	while (((groupMem=grp->gr_mem[i]) != NULL)) {
	    [returnArray addObject:[NSString stringWithUTF8String:groupMem]];
	    i++;
	}
    }
    endgrent();
    
    return [returnArray autorelease];
}
    
NSString *realNameForUser(NSString *userName)
{
    struct passwd *pw=NULL;
    
    pw=getpwnam([userName UTF8String]);
    
    if (pw != NULL) {
	return [NSString stringWithUTF8String:pw->pw_gecos];
    }
    
    return nil;
}

NSString *realNameForUID(int userID)
{
    struct passwd *pw=NULL;
    
    pw=getpwuid(userID);
    
    if (pw != NULL) {
	return [NSString stringWithUTF8String:pw->pw_gecos];
    }
    
    return nil;
}

NSDictionary *infoForUID(int userId)
{
    NSMutableDictionary *returnDict=[[NSMutableDictionary alloc]initWithCapacity:1];
    NSMutableArray *groupArray=[[NSMutableArray alloc]initWithCapacity:1];
    NSMutableArray *groupIDArray=[[NSMutableArray alloc]initWithCapacity:1];
    struct passwd *pw=NULL;
    struct group *grp=NULL;
    int i=0;
    
    if ((pw=getpwuid(userId)) != NULL) {
	char *grpName=NULL, *groupMem=NULL;
	grp=getgrgid(pw->pw_gid);
	if (grp != NULL) {
	    grpName=grp->gr_name;
	} else {
	    grpName="UNKNOWN GROUP";
	}
	
	[returnDict setObject:[NSNumber numberWithInt:pw->pw_uid] forKey:@"User ID"];
	[returnDict setObject:[NSString stringWithUTF8String:pw->pw_name] forKey:@"User Name"];
	[returnDict setObject:[NSString stringWithUTF8String:pw->pw_gecos] forKey:@"User Real Name"];
	[returnDict setObject:[NSString stringWithUTF8String:pw->pw_dir] forKey:@"User Home"];
	[returnDict setObject:[NSString stringWithUTF8String:pw->pw_shell] forKey:@"User Shell"];
	[returnDict setObject:[NSString stringWithUTF8String:grpName] forKey:@"User Group"];
	[returnDict setObject:[NSNumber numberWithInt:pw->pw_gid] forKey:@"User Group ID"];
	
	while ((grp=getgrent()) != NULL) {
	    i=0;
	    while (((groupMem=grp->gr_mem[i]) != NULL)) {
		if (!(strcmp(pw->pw_name, groupMem)) && (grp->gr_gid != pw->pw_gid)) {
		    [groupArray addObject:[NSString stringWithUTF8String:grp->gr_name]];
		    [groupIDArray addObject:[NSNumber numberWithInt:grp->gr_gid]];
		}
		i++;
	    }
	}
	[returnDict setObject:groupArray forKey:@"User Other Groups"];
	[returnDict setObject:groupIDArray forKey:@"User Other Group IDs"];
    }
    endgrent();
    endpwent();
    
    [groupArray release];
    [groupIDArray release];
    
    return [returnDict autorelease];
}

NSDictionary *infoForUser(NSString *name)
{
    NSMutableDictionary *returnDict=[[NSMutableDictionary alloc]initWithCapacity:1];
    NSMutableArray *groupArray=[[NSMutableArray alloc]initWithCapacity:1];
    NSMutableArray *groupIDArray=[[NSMutableArray alloc]initWithCapacity:1];
    struct passwd *pw=NULL;
    struct group *grp=NULL;
    int i=0;
    
    if ((pw=getpwnam([name UTF8String])) != NULL) {
	char *grpName=NULL, *groupMem=NULL;
	grp=getgrgid(pw->pw_gid);
	if (grp != NULL) {
	    grpName=grp->gr_name;
	} else {
	    grpName="UNKNOWN GROUP";
	}
	
	[returnDict setObject:[NSNumber numberWithInt:pw->pw_uid] forKey:@"User ID"];
	[returnDict setObject:[NSString stringWithUTF8String:pw->pw_name] forKey:@"User Name"];
	[returnDict setObject:[NSString stringWithUTF8String:pw->pw_gecos] forKey:@"User Real Name"];
	[returnDict setObject:[NSString stringWithUTF8String:pw->pw_dir] forKey:@"User Home"];
	[returnDict setObject:[NSString stringWithUTF8String:pw->pw_shell] forKey:@"User Shell"];
	[returnDict setObject:[NSString stringWithUTF8String:grpName] forKey:@"User Group"];
	[returnDict setObject:[NSNumber numberWithInt:pw->pw_gid] forKey:@"User Group ID"];
	
	while ((grp=getgrent()) != NULL) {
	    i=0;
	    while (((groupMem=grp->gr_mem[i]) != NULL)) {
		if (!(strcmp(pw->pw_name, groupMem)) && (grp->gr_gid != pw->pw_gid)) {
		    [groupArray addObject:[NSString stringWithUTF8String:grp->gr_name]];
		    [groupIDArray addObject:[NSNumber numberWithInt:grp->gr_gid]];
		}
		i++;
	    }
	}
	[returnDict setObject:groupArray forKey:@"User Other Groups"];
	[returnDict setObject:groupIDArray forKey:@"User Other Group IDs"];
    }
    
    endgrent();
    endpwent();
    
    [groupArray release];
    [groupIDArray release];
    
    return [returnDict autorelease];
}

NSArray *infoForAllUsers(void)
{
    NSMutableArray *returnArray=[[NSMutableArray alloc]initWithCapacity:1];
    NSMutableDictionary *aDict=[[NSMutableDictionary alloc]initWithCapacity:1];
    NSMutableArray *groupArray=[[NSMutableArray alloc]initWithCapacity:1];
    NSMutableArray *groupIDArray=[[NSMutableArray alloc]initWithCapacity:1];
    struct passwd *pw=NULL;
    struct group *grp=NULL;
    int i=0;
    
    while ((pw=getpwent()) != NULL) {
	char *grpName=NULL, *groupMem=NULL;
	grp=getgrgid(pw->pw_gid);
	if (grp != NULL) {
	    grpName=grp->gr_name;
	} else {
	    grpName="UNKNOWN GROUP";
	}
	
	[aDict setObject:[NSNumber numberWithInt:pw->pw_uid] forKey:@"User ID"];
	[aDict setObject:[NSString stringWithUTF8String:pw->pw_name] forKey:@"User Name"];
	[aDict setObject:[NSString stringWithUTF8String:pw->pw_gecos] forKey:@"User Real Name"];
	[aDict setObject:[NSString stringWithUTF8String:pw->pw_dir] forKey:@"User Home"];
	[aDict setObject:[NSString stringWithUTF8String:pw->pw_shell] forKey:@"User Shell"];
	[aDict setObject:[NSString stringWithUTF8String:grpName] forKey:@"User Group"];
	[aDict setObject:[NSNumber numberWithInt:pw->pw_gid] forKey:@"User Group ID"];
	
	while ((grp=getgrent()) != NULL) {
	    i=0;
	    while (((groupMem=grp->gr_mem[i]) != NULL)) {
		if (!(strcmp(pw->pw_name, groupMem)) && (grp->gr_gid != pw->pw_gid)) {
		    [groupArray addObject:[NSString stringWithUTF8String:grp->gr_name]];
		    [groupIDArray addObject:[NSNumber numberWithInt:grp->gr_gid]];
		}
		i++;
	    }
	}
	
	[aDict setObject:groupArray forKey:@"User Other Groups"];
	[aDict setObject:groupIDArray forKey:@"User Other Group IDs"];
	
	[groupArray release]; groupArray=[[NSMutableArray alloc]initWithCapacity:1];
	[groupIDArray release]; groupIDArray=[[NSMutableArray alloc]initWithCapacity:1];
	
	[returnArray addObject:aDict];
	
	[aDict release];
	aDict=[[NSMutableDictionary alloc]initWithCapacity:1];
    }
    endgrent();
    endpwent();
    
    return [returnArray autorelease];
}
